package com.install4j.api.beaninfo;

import java.beans.IntrospectionException;

/**
 * Property descriptor for properties that contain a list of configurable actions.
 * The type of the property must by {@link com.install4j.api.beans.ActionList}.
 * <p>Using this class is not strictly required. In principle, you could also set values for the {@code ATTRIBUTE_*} constants in the property descriptor
 * of an unrelated property descriptor class.
 * @see com.install4j.api.beans.ActionList
 */
public class ActionListPropertyDescriptor extends Install4JPropertyDescriptor {

    /**
     * @see #setAllowedActionClasses(Class[])
     */
    public static final String ATTRIBUTE_ALLOWED_ACTION_CLASSES = "allowedActions";

    /**
     * @see #setAllowedActionClasses(Class[])
     */
    public static final String ATTRIBUTE_ACTION_KEYS = "actionsKeys";

    /**
     * Action key for properties of type {@link com.install4j.api.beans.ActionList} that do not support rollback.
     * Properties in standard actions that deal with rollback are not shown in the install4j IDE in that case.
     * Call {@link #setActionKeys(String[])} with an array that contains this string to select this behavior for
     * an action list property.
     */
    public static final String ACTION_KEY_NO_ROLLBACK = "noRollback";

    /**
     * Create a property descriptor that can be passed to {@link Install4JBeanInfo#addPropertyDescriptor(Install4JPropertyDescriptor)}.
     * @param propertyName the name of the property
     * @param beanClass the class of the bean that contains the property. <b>Note:</b> This is <b>not</b> the class of the property.
     * @param displayName the display name of the property
     * @param shortDescription a short description of the property in HTML format. You do not have to start the description with &lt;html&gt;, it will be prepended automatically.
     * @return the property descriptor
     */
    public static ActionListPropertyDescriptor create(String propertyName, Class beanClass, String displayName, String shortDescription) {
        try {
            ActionListPropertyDescriptor descriptor = ActionListPropertyDescriptor.createDescriptor(propertyName, beanClass);
            descriptor.setDisplayName(displayName);
            descriptor.setShortDescription(shortDescription);
            return descriptor;
        } catch (IntrospectionException e) {
            throw new RuntimeException(e);
        }
    }

    private static ActionListPropertyDescriptor createDescriptor(String propertyName, Class beanClass) throws IntrospectionException {
        try {
            return new ActionListPropertyDescriptor(propertyName, beanClass);
        } catch (Exception ex) {
        }
        return new ActionListPropertyDescriptor(propertyName, beanClass, "get" + capitalize(propertyName), null);
    }

    ActionListPropertyDescriptor(String propertyName, Class beanClass) throws IntrospectionException {
        super(propertyName, beanClass);
    }

    ActionListPropertyDescriptor(String propertyName, Class beanClass, String readMethod, String writeMethod) throws IntrospectionException {
        super(propertyName, beanClass, readMethod, writeMethod);
    }

    /**
     * If set to a non-null value, the action registry dialog will only display the specified actions.
     * Derived classes are also shown, so you can include entire hierarchies with a single class.
     * Otherwise all actions will be offered.
     * @param actionClasses an array with the action classes
     * @return {@code this}, for chained calls on this property descriptor
     */
    public ActionListPropertyDescriptor setAllowedActionClasses(Class[] actionClasses) {
        if (actionClasses != null) {
            setValue(ATTRIBUTE_ALLOWED_ACTION_CLASSES, actionClasses);
        }
        return this;
    }

    /**
     * Action keys can be used to show or hide properties of other actions.
     * See {@link Install4JPropertyDescriptor#setActionListShownKeys(String[])} and
     * {@link Install4JPropertyDescriptor#setActionListHiddenKeys(String[])} for more information.
     * @param actionsKeys an array with the action keys
     * @return {@code this}, for chained calls on this property descriptor
     */
    public ActionListPropertyDescriptor setActionKeys(String[] actionsKeys) {
        if (actionsKeys != null) {
            setValue(ATTRIBUTE_ACTION_KEYS, actionsKeys);
        }
        return this;
    }
}
