package com.install4j.api.launcher;

import com.exe4j.Controller;
import com.install4j.runtime.installer.helper.InstallerUtil;

import java.lang.reflect.InvocationTargetException;
import java.util.EventListener;

/**
 *
 * <p>This class allows you to register a listener to receive startup
 * events in single instance mode on Microsoft Windows and
 * file open events on macOS.</p>
 *
 * On macOS, this is a facade for the platform-specific Java extensions.
 * If you want to use additional features, please use the com.apple.eawt
 * classes directly that are contained in all macOS JREs. For detailed functionality
 * please refer to <a href="http://www.coderanch.com/how-to/javadoc/appledoc/api/com/apple/eawt/OpenFilesHandler.html" target="_top">the Javadoc</a>.
 *
 * @author ej-technologies GmbH
 *
 **/
public class StartupNotification {
    private static Listener listener;

    /**
     * Register a listener to receive startup events in single instance mode on
     * Microsoft Windows or to receive file open events on macOS.
     * Request that are already queued may be invoked immediately on the startup listener before this call returns.
     * Subsequent calls will be from different threads.
     * @param listener the listener
     */
    public static void registerStartupListener(final Listener listener) {
        if (InstallerUtil.isWindows()) {
            StartupNotification.listener = listener;
            if (listener != null) {
                Controller.registerStartupListener(new ListenerAdapter());
            }
        } else if (InstallerUtil.isMacOS()) {
            try {
                Class.forName("com.install4j.runtime.installer.platform.macos.MacFileAssociations").getMethod("setStartupListener",
                        new Class[] { Listener.class }).invoke(null, listener);
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }
        }
    }


    /**
     * Sets if the Quit command should be handled on macOS (that means if the application
     * should quit when the command is invoked). This is true be default.
     * @param handleQuit true if quit should be handled
     */
    public static void setHandleQuit(boolean handleQuit) {
        if (InstallerUtil.isMacOS()) {
            // reflection needed for idea
            try {
                Class.forName("com.install4j.runtime.installer.platform.macos.MacFileAssociations").getMethod("setHandleQuit",
                        new Class[] { boolean.class }).invoke(null, handleQuit);
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }
        }
    }


    /**
     * The interface for startup events in single instance mode or for open file events on macOS.
     *
     * @author ej-technologies GmbH
     *
    */
    public interface Listener extends EventListener {
        /**
         * Will be called when the executable is started again or when a file open event is received.
         * Note that each invocation may be from a separate thread. If you have to serialize
         * calls into this method, please synchronize the implementation.
         * @param parameters the parameters of the invocation, or the file to open on macOS.
         * For multiple files, files are surrounded by double-quotes and separated by spaces.
         *  <p><b>Note:</b> any default arguments defined for the launcher will not be
         *  passed again here</p>.
         */
        void startupPerformed(String parameters);
    }

    private static class ListenerAdapter implements Controller.StartupListener {
        @Override
        public void startupPerformed(String parameters) {
            if (listener != null) {
                listener.startupPerformed(parameters);
            }
        }
    }
}
